package reader;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;

import reader.structure.Block;
import reader.structure.BlockType;
import reader.structure.Thread;
import reader.structure.ThreadTool;

public class Reader {
	private String principalThreadID;

	private Long initialTime;

	private Long finalTime;

	private ThreadTool threadTool;
	
	public ThreadTool readFile(String fileName) {
		threadTool = new ThreadTool();
		File file = new File(fileName);
		try {
			FileInputStream fileInputStream = new FileInputStream(file);
			DataInputStream dataInputStream = new DataInputStream(
					fileInputStream);
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					dataInputStream));

			String line;

			while ((line = reader.readLine()) != null) {
				this.parseLine(line);
			}

			dataInputStream.close();
			
			this.threadTool.cleanWaitTimes();
			this.threadTool.adjustMutexes();
			
			printStructure(this.threadTool.getThreadList());

		} catch (FileNotFoundException e) {
			System.out.println("Arquivo nao encontrado.");
			System.exit(-2);
		} catch (IOException e) {
			System.out.println("Problemas na leitura do arquivo de entrada.");
			System.exit(-3);
		}
		return this.threadTool;
	}

	private void printStructure(List<Thread> threadList) {
		for(Thread thread:threadList) {
			System.out.println("Thread ID: " + thread.getId() + " Tempo inicial: " + thread.getBegin() + " Tempo final: " + thread.getDuration());
			for(Block block:thread.getBlockList()){
				System.out.println("Tipo do bloco: " + block.getBlockType() + " Inicio: " + block.getBegin() + " Fim: " + block.getEnd());
			}
		}
	}

	private void parseLine(String line) {
		String[] data = line.split(";");
		int index;
		int size;

		switch (Integer.parseInt(data[0])) {
		case 1:
			this.threadTool.addThread(new reader.structure.Thread(data[1], Long.parseLong(data[2])));
			break;
		case 2:
			this.threadTool.addBlock(new Block(BlockType.WAIT, Long.parseLong(data[3]), Long.parseLong(data[5])), data[1]);
			break;
		case 3:
			index = this.threadTool.getThreadList().indexOf(new reader.structure.Thread(data[1], -1)); // -1 pq a thread provavelmente nao foi finalizada.
			size = this.threadTool.getThreadList().get(index).getBlockList().size();
			long initialTime = this.threadTool.getThreadList().get(index).getBlockList().get(size - 1).getEnd();
			this.threadTool.addBlock(new Block(BlockType.MUTEX, initialTime, Long.parseLong(data[3])), data[1]);
			break;
		case 4:
			// Desabilitado temporariamente.
			// Utilização de join da thread.
			break;
		case 5:
			this.threadTool.addBlock(new Block(BlockType.CODE, Long.parseLong(data[2]), Long.parseLong(data[3])), data[1]);
		}
	}

	public Long getFinalTime() {
		return this.finalTime;
	}

	public Long getInitialTime() {
		return this.initialTime;
	}

	public String getPrincipalThreadID() {
		return this.principalThreadID;
	}
}
