package reader.applet;

import java.awt.*;
import java.util.Collections;
import java.util.List;

import reader.Reader;
import reader.structure.Block;
import reader.structure.Thread;
import reader.structure.ThreadTool;

public class ThreadToolApplet extends java.applet.Applet {

	private String title = "Thread Tool Applet";
	private Font font;
	private FontMetrics metrics;

	private String labels[];

	private int scale = 5;
	private int maxLabelWidth = 0;
	private int barSpacing = 30;
	private long maxValue = 0;

	private Reader reader;
	private ThreadTool threadTool;

	public void init() {
		reader = new Reader();
		this.threadTool = reader.readFile("../files/input.txt");

		font = new java.awt.Font("Courier", Font.BOLD, 12);
		metrics = getFontMetrics(font);

		for (Thread thread : this.threadTool.getThreadList()) {
			maxValue = Math.max(maxValue, thread.getDuration());
		}
		parseLabel();
	}

	private void parseLabel() {
		int i = 0;
		labels = new String[this.threadTool.getThreadListSize()];
		for (Thread thread : this.threadTool.getThreadList()) {
			labels[i] = "Thread" + thread.getId();
			i++;
		}
		for (i = 0; i < this.threadTool.getThreadListSize(); i++) {
			maxLabelWidth = Math.max(metrics.stringWidth((String) (labels[i])),
					maxLabelWidth);
		}
	}

	public void paint(Graphics g) {
		// draw the title centered at the bottom of the bar graph
		g.setColor(Color.black);
		g.setFont(font);

		g.drawRect(0, 0, getSize().width - 1, getSize().height - 1);

		int titleWidth = metrics.stringWidth(title);
		int cx = Math.max((getSize().width - titleWidth) / 2, 0);
		int cy = getSize().height - metrics.getDescent();
		g.drawString(title, cx, cy);

		paintChart(g);
	}

	private void paintChart(Graphics g) {
		int barWidth = maxLabelWidth;

		int listSize = this.threadTool.getThreadListSize();
		for (int i = 0; i < listSize; i++) {
			// X coordinate for this label and bar (centered)
			int widthOfItems = (barWidth + barSpacing) * listSize - barSpacing;
			int cx = Math.max((getSize().width - widthOfItems) / 2, 0);
			cx += (maxLabelWidth + barSpacing) * i;

			// Y coordinate for this label and bar
			int cy = getSize().height - metrics.getHeight()
					- metrics.getDescent() - 4;

			// draw the label
			g.setColor(Color.black);
			g.drawString(labels[i], cx, cy);
			cy -= metrics.getHeight() - 3;
			
			
			List<Block> blockList = this.threadTool.getThreadList().get(i)
					.getBlockList();
			
			Collections.sort(blockList);
			
			Block block = null;
			for (int j = 0; j < blockList.size(); j++) {
				block = blockList.get(j);
				g.setColor(block.getColor());
				g.fillRect(cx, cy - ((int) block.getEnd() * scale),
						barWidth + 1, (((int) block.getEnd() - (int) block
								.getBegin()) * scale) + 1);
				g.setColor(Color.BLACK);
				g.drawString(Long.toString(block.getEnd()), cx + barWidth, cy - (int)((block.getEnd() * scale)));
			}

			
			cy -= ((int) block.getEnd() * scale) + 5;
			g.setColor(Color.black);
			// draw the value on top of the bar
			g.setColor(g.getColor().darker());
			g.drawString(Long.toString(this.threadTool.getThreadList().get(i)
					.getDuration()), cx, cy);
		}
	}
}