package reader.structure;

import java.util.ArrayList;
import java.util.List;

public class Thread implements Comparable {
	private String id;

	private List<Block> blockList = new ArrayList<Block>();

	private long begin;

	public Thread(String id, long begin) {
		super();
		this.id = id;
		this.begin = begin;
	}

	public String getId() {
		return this.id;
	}

	public long getBegin() {
		return begin;
	}
	
	public void setBegin(long begin) {
		this.begin = begin;
	}
	
	public List<Block> getBlockList() {
		return this.blockList;
	}

	public void addBlock(Block block) {
		if (this.blockList.size() > 0) {
			Block lastBlock = this.blockList.get(this.blockList.size() - 1);
			block.setBegin(lastBlock.getEnd());
		}
		this.blockList.add(block);
		block.setThread(this);
	}

	public void putDelay(int i, long delay) {
		Block block = this.blockList.get(i);
		Block waitBlock = new Block(BlockType.WAIT, block.getBegin(), block.getBegin() + delay);
		this.blockList.add(i, waitBlock);
		waitBlock.setThread(this);
		this.adjustTimes();
	}

	public void moveBlock(int i, Block block) {
		if (this.blockList.indexOf(block) < i) {
			this.blockList.remove(block);
			this.blockList.add(i - 1, block);
		} else {
			this.blockList.remove(block);
			this.blockList.add(i, block);
		}
		this.adjustTimes();
	}

	public void adjustTimes() {
		Block currentBlock;
		Block previousBlock = this.blockList.get(0);
		previousBlock.setBegin(this.begin);

		for (int i = 1; i < this.blockList.size(); i++) {
			currentBlock = this.blockList.get(i);
			currentBlock.setBegin(previousBlock.getEnd());
			previousBlock = currentBlock;
		}
	}

	public void cleanThread() {
		List<Block> blockForRemotion = new ArrayList<Block>();
		
		for (Block block : this.blockList) {
			if (block.getBlockType() == BlockType.WAIT) {
				blockForRemotion.add(block);
			}
		}
		
		this.blockList.removeAll(blockForRemotion);
		
		this.adjustTimes();
	}
	
	public int compareTo(Object o) {
		if (o instanceof Thread) {
			Thread other = (Thread) o;
			return this.id.compareTo(other.getId());
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Thread) {
			Thread other = (Thread) obj;
			return this.compareTo(other) == 0;
		} else {
			return false;
		}
	}

	public long getDuration() {
		Block lastBlock = this.blockList.get(this.blockList.size() - 1);
		return lastBlock.getEnd() - this.begin;
	}
}
